<?php

namespace Drupal\file_temporary_validator\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;
use Drupal\file_temporary_validator\Service\FileTemporaryValidator;

/**
 * Validates the duplicate_upload_validator constraint.
 */
final class DuplicateUploadValidatorConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {
  use StringTranslationTrait;

  /**
   * Constructs the DuplicateUploadValidatorConstraintValidator.
   *
   * @param \Drupal\file_temporary_validator\Service\FileTemporaryValidator $validator
   *   The file temporary validator service.
   */
  public function __construct(
    private readonly FileTemporaryValidator $validator,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static($container->get('file_temporary_validator'));
  }

  /**
   * {@inheritdoc}
   */
  public function validate(mixed $value, Constraint $constraint): void {
    if (!$constraint instanceof DuplicateUploadValidatorConstraint) {
      throw new UnexpectedTypeException($constraint, DuplicateUploadValidatorConstraint::class);
    }
    if (!$value instanceof FileInterface) {
      return;
    }

    if ($duplicate = $this->validator->getFileDuplications($value)) {
      $violation = $this->validator->buildDuplicateViolation($value, $duplicate);
      $message = $violation->getUntranslatedString();
      $parameters = $violation->getArguments();
      $this->context->buildViolation($message, $parameters)->addViolation();
    }
  }

}
